package ca.odell.glazedlists.swing;

import ca.odell.glazedlists.*;
import ca.odell.glazedlists.impl.swing.*;
import javax.swing.BoundedRangeModel;

/**
 * A factory for creating all sorts of objects to be used with Glazed Lists.
 *
 * @author <a href="mailto:jesse@odel.on.ca">Jesse Wilson</a>
 */
public final class GlazedListsSwing {
    
    /**
     * A dummy constructor to prevent instantiation of this class
     */
    private GlazedListsSwing() {
        super();
        throw new UnsupportedOperationException();
    }
    
    /**
     * Wraps the source in an {@link EventList} that fires all of its update events
     * from the Swing event dispatch thread.
     */
    public static TransformedList swingThreadProxyList(EventList source) {
        return new SwingThreadProxyEventList(source);
    }
    
    /**
     * Creates a model that manipulates the lower bound of the specified
     * ThresholdList.  The ThresholdList linked to this model type will contain
     * a range of Objects between the results of getValue() and getMaximum()
     * on the BoundedRangeModel.
     */
    public static BoundedRangeModel lowerRangeModel(ThresholdList target) {
        return new LowerThresholdRangeModel(target);
    }
    
    /**
     * Creates a model that manipulates the upper bound of the specified
     * ThresholdList.  The ThresholdList linked to this model type will contain
     * a range of Objects between the results of getMinimum() and getValue()
     * on the BoundedRangeModel.
     */
    public static BoundedRangeModel upperRangeModel(ThresholdList target) {
        return new UpperThresholdRangeModel(target);
    }
}
