package ca.odell.glazedlists;

import java.util.*;
import junit.framework.*;

/**
 * A CompositeListTest tests the functionality of the CompositeList.
 *
 * @author <a href="mailto:jesse@odel.on.ca">Jesse Wilson</a>
 */
public class CompositeListTest extends TestCase {
    
    public CompositeListTest() {
        super();
    }
    
    /**
     * Verifies that the a single source works.
     */
    public void testSingleSource() {
        EventList wendys = new BasicEventList();
        wendys.add("Classic Single");
        wendys.add("Chili");
        wendys.add("Frosty");
        wendys.add("Junior Bacon Cheeseburger");
        CompositeList fastFood = new CompositeList();
        fastFood.addListEventListener(new ListConsistencyListener(fastFood, "fastFood", false));
        fastFood.addMemberList(wendys);
        assertEquals(wendys, fastFood);
        wendys.add("Sour Cream \'n\' Onion Baked Potato");
        wendys.add("Taco Supremo Salad");
        assertEquals(wendys, fastFood);
        wendys.remove(1);
        fastFood.set(0, "Big Bacon Classic");
        fastFood.remove(1);
        assertEquals(wendys, fastFood);
    }
    
    /**
     * Verifies that multiple sources work.
     */
    public void testMultipleSources() {
        List fastFoodVerify = new ArrayList();
        EventList wendys = new BasicEventList();
        wendys.add("Classic Single");
        wendys.add("Chili");
        wendys.add("Frosty");
        wendys.add("Junior Bacon Cheeseburger");
        EventList mcDonalds = new BasicEventList();
        mcDonalds.add("McDLT");
        mcDonalds.add("McPizza");
        mcDonalds.add("McSalad Shaker");
        mcDonalds.add("Royal with Cheese");
        EventList tacoBell = new BasicEventList();
        tacoBell.add("Fries Supreme");
        tacoBell.add("Bean Burrito");
        CompositeList fastFood = new CompositeList();
        fastFood.addListEventListener(new ListConsistencyListener(fastFood, "fastFood", false));
        fastFood.addMemberList(wendys);
        fastFood.addMemberList(mcDonalds);
        fastFood.addMemberList(tacoBell);
        fastFoodVerify.clear();
        fastFoodVerify.addAll(wendys);
        fastFoodVerify.addAll(mcDonalds);
        fastFoodVerify.addAll(tacoBell);
        assertEquals(fastFoodVerify, fastFood);
        wendys.add("Sour Cream \'n\' Onion Baked Potato");
        wendys.add("Taco Supremo Salad");
        fastFoodVerify.clear();
        fastFoodVerify.addAll(wendys);
        fastFoodVerify.addAll(mcDonalds);
        fastFoodVerify.addAll(tacoBell);
        assertEquals(fastFoodVerify, fastFood);
        wendys.remove(1);
        fastFood.set(0, "Big Bacon Classic");
        fastFood.remove(1);
        fastFoodVerify.clear();
        fastFoodVerify.addAll(wendys);
        fastFoodVerify.addAll(mcDonalds);
        fastFoodVerify.addAll(tacoBell);
        assertEquals(fastFoodVerify, fastFood);
        mcDonalds.add("Big Mac");
        fastFoodVerify.clear();
        fastFoodVerify.addAll(wendys);
        fastFoodVerify.addAll(mcDonalds);
        fastFoodVerify.addAll(tacoBell);
        assertEquals(fastFoodVerify, fastFood);
        fastFood.removeMemberList(mcDonalds);
        fastFoodVerify.clear();
        fastFoodVerify.addAll(wendys);
        fastFoodVerify.addAll(tacoBell);
        assertEquals(fastFoodVerify, fastFood);
    }
    
    /**
     * Verifies that remove member list does so by reference.
     */
    public void testRemoveByReference() {
        EventList wendys = new BasicEventList();
        EventList mcDonalds = new BasicEventList();
        EventList tacoBell = new BasicEventList();
        CompositeList fastFood = new CompositeList();
        fastFood.addListEventListener(new ListConsistencyListener(fastFood, "fastFood", false));
        fastFood.addMemberList(wendys);
        fastFood.addMemberList(mcDonalds);
        fastFood.addMemberList(tacoBell);
        fastFood.removeMemberList(tacoBell);
        fastFood.removeMemberList(wendys);
        assertEquals(mcDonalds, fastFood);
        mcDonalds.add("Arch Deluxe");
        assertEquals(mcDonalds, fastFood);
    }
    
    /**
     * Verifies that multiple copies of the same list can  be added.
     */
    public void testMultipleCopies() {
        List fastFoodVerify = new ArrayList();
        EventList wendys = new BasicEventList();
        wendys.add("Spicy Chicken Sandwich");
        EventList mcDonalds = new BasicEventList();
        mcDonalds.add("Arch Deluxe");
        mcDonalds.add("McLean Deluxe");
        CompositeList fastFood = new CompositeList();
        fastFood.addMemberList(wendys);
        fastFood.addMemberList(mcDonalds);
        fastFoodVerify.clear();
        fastFoodVerify.addAll(wendys);
        fastFoodVerify.addAll(mcDonalds);
        assertEquals(fastFoodVerify, fastFood);
        fastFood.addMemberList(wendys);
        fastFood.addMemberList(mcDonalds);
        fastFoodVerify.clear();
        fastFoodVerify.addAll(wendys);
        fastFoodVerify.addAll(mcDonalds);
        fastFoodVerify.addAll(wendys);
        fastFoodVerify.addAll(mcDonalds);
        assertEquals(fastFoodVerify, fastFood);
        fastFood.removeMemberList(wendys);
        fastFood.removeMemberList(wendys);
        fastFoodVerify.clear();
        fastFoodVerify.addAll(mcDonalds);
        fastFoodVerify.addAll(mcDonalds);
        assertEquals(fastFoodVerify, fastFood);
    }
    
    /**
     * Test that {@link CompositeList} is well behaved when only a single element
     * is removed.
     */
    public void testSingleElements() {
        EventList alpha = new BasicEventList();
        alpha.add("A");
        EventList beta = new BasicEventList();
        beta.add("B");
        CompositeList aToB = new CompositeList();
        aToB.addListEventListener(new ListConsistencyListener(aToB, "AtoB", false));
        aToB.addMemberList(alpha);
        aToB.removeMemberList(alpha);
        aToB.addMemberList(beta);
    }
}
