package ca.odell.glazedlists;

import ca.odell.glazedlists.matchers.*;
import ca.odell.glazedlists.impl.matchers.NotMatcher;
import junit.framework.TestCase;

public class MatcherTest extends TestCase {
    
    public MatcherTest() {
        super();
    }
    
    public void testTrueMatcher() {
        Matcher s = Matchers.trueMatcher();
        Matcher b = Matchers.trueMatcher();
        assertTrue((Object)s == b);
    }
    
    public void testFalseMatcher() {
        Matcher s = Matchers.falseMatcher();
        Matcher b = Matchers.falseMatcher();
        assertTrue((Object)s == b);
    }
    
    public void testNotMatcher() {
        Matcher s = new NotMatcher(new CapitalizedStringMatcher());
        assertFalse(s.matches("James"));
        assertTrue(s.matches("james"));
        Matcher b = new NotMatcher(new OnMatcher());
        assertFalse(b.matches(Boolean.TRUE));
        assertTrue(b.matches(Boolean.FALSE));
    }
    
    public void testGenerics() {
        NumberMatcherEditor numMatcherEditor = new NumberMatcherEditor();
        assertTrue(numMatcherEditor.getMatcher().matches(new Integer(5)));
        numMatcherEditor.setNumber(new Integer(10));
        assertFalse(numMatcherEditor.getMatcher().matches(new Integer(5)));
        assertTrue(numMatcherEditor.getMatcher().matches(new Integer(10)));
        numMatcherEditor.setNumber(new Float(3.14F));
        assertFalse(numMatcherEditor.getMatcher().matches(new Integer(10)));
        assertTrue(numMatcherEditor.getMatcher().matches(new Float(3.14F)));
        MatcherEditor typedMatcherEditor = numMatcherEditor;
        assertFalse(typedMatcherEditor.getMatcher().matches(new Integer(10)));
        assertTrue(typedMatcherEditor.getMatcher().matches(new Float(3.14F)));
        typedMatcherEditor = new TextMatcherEditor(GlazedLists.toStringTextFilterator());
        assertTrue(typedMatcherEditor.getMatcher().matches(new Integer(10)));
        assertTrue(typedMatcherEditor.getMatcher().matches(new Float(3.14F)));
        ((TextMatcherEditor)(TextMatcherEditor)typedMatcherEditor).setFilterText(new String[]{"3"});
        assertFalse(typedMatcherEditor.getMatcher().matches(new Integer(10)));
        assertTrue(typedMatcherEditor.getMatcher().matches(new Float(3.14F)));
    }
    
    private class NumberMatcherEditor extends AbstractMatcherEditor {
        
        private NumberMatcherEditor() {
            super();
        }
        
        public void setNumber(Number number) {
            this.fireChanged(new NumberMatcher(number));
        }
    }
    
    private class NumberMatcher implements Matcher {
        private final Number value;
        
        public NumberMatcher(Number value) {
            super();
            this.value = value;
        }
        
        public boolean matches(Object x0) {
            Number item = (Number)x0;
            return value.doubleValue() == item.doubleValue();
        }
        /*missing*/
    }
    
    private class CapitalizedStringMatcher implements Matcher {
        
        private CapitalizedStringMatcher() {
            super();
        }
        
        public boolean matches(Object x0) {
            String item = (String)x0;
            return item != null && item.length() > 0 && Character.isUpperCase(item.charAt(0));
        }
        /*missing*/
    }
    
    private class OnMatcher implements Matcher {
        
        private OnMatcher() {
            super();
        }
        
        public boolean matches(Object x0) {
            Boolean item = (Boolean)x0;
            return item != null && item.booleanValue();
        }
        /*missing*/
    }
}
