package ca.odell.glazedlists;

import junit.framework.*;
import java.util.*;
import ca.odell.glazedlists.matchers.Matchers;

/**
 * This test verifies that the toString() method on all lists is consistent
 * with the toString() method of {@link ArrayList}.
 *
 * @author <a href="mailto:jesse@odel.on.ca">Jesse Wilson</a>
 */
public class ToStringTest extends TestCase {
    
    public ToStringTest() {
        super();
    }
    
    /**
     * Validate that toString() is consistent with ArrayList on all lists.
     */
    public void testToStringConsistency() {
        ArrayList controlList = new ArrayList();
        BasicEventList basicEventList = new BasicEventList();
        FilterList filterList = new FilterList(basicEventList, Matchers.trueMatcher());
        SortedList sortedList = new SortedList(basicEventList);
        assertEquals(controlList.toString(), basicEventList.toString());
        assertEquals(controlList.toString(), filterList.toString());
        assertEquals(controlList.toString(), sortedList.toString());
        controlList.add("Apple");
        basicEventList.add("Apple");
        assertEquals(controlList.toString(), basicEventList.toString());
        assertEquals(controlList.toString(), filterList.toString());
        assertEquals(controlList.toString(), sortedList.toString());
        controlList.add("Banana");
        controlList.add("Cherry");
        controlList.add("Donut");
        basicEventList.add("Banana");
        basicEventList.add("Cherry");
        basicEventList.add("Donut");
        assertEquals(controlList.toString(), basicEventList.toString());
        assertEquals(controlList.toString(), filterList.toString());
        assertEquals(controlList.toString(), sortedList.toString());
    }
}
