package ca.odell.glazedlists;

import junit.framework.*;
import java.util.*;

/**
 * A UniqueListTest tests the functionality of the UniqueList
 *
 * @author <a href="mailto:kevin@swank.ca">Kevin Maltby</a>
 */
public class UniqueListTest extends TestCase {
    
    public UniqueListTest() {
        super();
    }
    private UniqueList unique = null;
    private BasicEventList source = null;
    
    /**
     * Prepare for the test.
     */
    public void setUp() {
        source = new BasicEventList();
        unique = new UniqueList(source);
    }
    
    /**
     * Clean up after the test.
     */
    public void tearDown() {
        unique = null;
        source = null;
    }
    
    /**
     * Testing add() with an empty source list 
     */
    public void testSimpleInsert() {
        source.add("A");
        assertEquals(1, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
    }
    
    public void testSortedNonDuplicateInsert() {
        source.add("A");
        source.add("B");
        source.add("C");
        assertEquals(3, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
    }
    
    public void testUnsortedNonDuplicateInsert() {
        source.add("C");
        source.add("A");
        source.add("B");
        assertEquals(3, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
    }
    
    public void testSimpleDuplicateInsert() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("A");
        assertEquals(1, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
    }
    
    public void testMultipleDuplicateInserts() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("C");
        source.add("C");
        source.add("C");
        assertEquals(2, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("C", (String)(String)unique.get(1));
    }
    
    /**
     * Testing add() with a non-empty source list 
     */
    public void testSimpleNonEmptySource() {
        unique = null;
        source.add("A");
        unique = new UniqueList(source);
        assertEquals(1, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
    }
    
    public void testSortedNonDuplicateNonEmptySource() {
        unique = null;
        source.add("A");
        source.add("B");
        source.add("C");
        unique = new UniqueList(source);
        assertEquals(3, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
    }
    
    public void testUnsortedNonDuplicateNonEmptySource() {
        unique = null;
        source.add("C");
        source.add("A");
        source.add("B");
        unique = new UniqueList(source);
        assertEquals(3, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
    }
    
    public void testSimpleDuplicateInNonEmptySource() {
        unique = null;
        source.add("A");
        source.add("A");
        source.add("A");
        unique = new UniqueList(source);
        assertEquals(1, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
    }
    
    public void testMultipleDuplicatesInNonEmptySource() {
        unique = null;
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("C");
        source.add("C");
        source.add("C");
        unique = new UniqueList(source);
        assertEquals(2, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("C", (String)(String)unique.get(1));
    }
    
    public void testSimpleAddToEndOfSortedNonDuplicateNonEmptySource() {
        unique = null;
        source.add("A");
        source.add("B");
        source.add("C");
        unique = new UniqueList(source);
        source.add("D");
        assertEquals(4, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
        assertEquals("D", (String)(String)unique.get(3));
    }
    
    public void testSimpleAddToFrontOfSortedNonDuplicateNonEmptySource() {
        unique = null;
        source.add("B");
        source.add("C");
        source.add("D");
        unique = new UniqueList(source);
        source.add("A");
        assertEquals(4, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
        assertEquals("D", (String)(String)unique.get(3));
    }
    
    public void testSimpleAddToUnsortedNonDuplicateNonEmptySource() {
        unique = null;
        source.add("D");
        source.add("A");
        source.add("C");
        unique = new UniqueList(source);
        source.add("B");
        assertEquals(4, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
        assertEquals("D", (String)(String)unique.get(3));
    }
    
    public void testAddingSimpleDuplicateToNonEmptySource() {
        unique = null;
        source.add("A");
        source.add("A");
        source.add("A");
        unique = new UniqueList(source);
        source.add("C");
        source.add("C");
        source.add("C");
        assertEquals(2, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("C", (String)(String)unique.get(1));
    }
    
    public void testAddingMultipleDuplicatesToNonEmptySource() {
        unique = null;
        source.add("C");
        source.add("C");
        source.add("C");
        source.add("A");
        source.add("A");
        source.add("A");
        unique = new UniqueList(source);
        source.add("D");
        source.add("D");
        source.add("D");
        source.add("B");
        source.add("B");
        source.add("B");
        assertEquals(4, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
        assertEquals("D", (String)(String)unique.get(3));
    }
    
    /**
     * Testing response to addAll() 
     */
    public void testSimpleAddOfSortedCollection() {
        LinkedList duplicates = new LinkedList();
        duplicates.add("A");
        duplicates.add("B");
        duplicates.add("C");
        source.addAll(duplicates);
        assertEquals(3, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
    }
    
    public void testSimpleAddOfUnsortedCollection() {
        LinkedList duplicates = new LinkedList();
        duplicates.add("B");
        duplicates.add("C");
        duplicates.add("A");
        source.addAll(duplicates);
        assertEquals(3, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
    }
    
    public void testAddOfCollectionContainingDuplicates() {
        LinkedList duplicates = new LinkedList();
        duplicates.add("A");
        duplicates.add("C");
        duplicates.add("A");
        duplicates.add("C");
        duplicates.add("A");
        duplicates.add("C");
        source.addAll(duplicates);
        assertEquals(2, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("C", (String)(String)unique.get(1));
    }
    
    public void testAddingCollectionDuplicatingContentsOfNonEmptySource() {
        unique = null;
        source.add("A");
        source.add("B");
        source.add("C");
        unique = new UniqueList(source);
        LinkedList duplicates = new LinkedList();
        duplicates.add("A");
        duplicates.add("B");
        duplicates.add("C");
        source.addAll(duplicates);
        assertEquals(3, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
    }
    
    public void testAddingCollectionOfSingleUniqueToNonEmptySource() {
        unique = null;
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("B");
        source.add("C");
        source.add("C");
        source.add("C");
        unique = new UniqueList(source);
        LinkedList duplicates = new LinkedList();
        duplicates.add("B");
        source.addAll(duplicates);
        assertEquals(3, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
    }
    
    public void testAddingCollectionWithNewDuplicatesToNonEmptySource() {
        unique = null;
        source.add("A");
        source.add("A");
        source.add("C");
        source.add("A");
        source.add("C");
        source.add("C");
        unique = new UniqueList(source);
        LinkedList duplicates = new LinkedList();
        duplicates.add("B");
        duplicates.add("B");
        duplicates.add("B");
        duplicates.add("A");
        duplicates.add("C");
        source.addAll(duplicates);
        assertEquals(3, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
    }
    
    public void testAddingCollectionOfDuplicatesToNonEmptySource() {
        unique = null;
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("B");
        source.add("C");
        source.add("C");
        source.add("C");
        unique = new UniqueList(source);
        LinkedList duplicates = new LinkedList();
        duplicates.add("B");
        duplicates.add("B");
        duplicates.add("B");
        duplicates.add("A");
        duplicates.add("C");
        source.addAll(duplicates);
        assertEquals(3, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
    }
    
    /**
     * Testing response to REMOVE event 
     */
    public void testSimpleRemoveByIndex() {
        source.add("A");
        source.remove(0);
        assertEquals(0, unique.size());
    }
    
    public void testRemoveOfSingleDuplicateByIndex() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.remove(2);
        assertEquals(1, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
    }
    
    public void testRemoveOfDuplicatesByIndex() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.remove(2);
        source.remove(1);
        assertEquals(1, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
    }
    
    public void testRemoveWithMultipleDuplicatesByIndex() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("B");
        source.add("C");
        source.add("C");
        source.add("C");
        source.remove(6);
        source.remove(5);
        source.remove(2);
        source.remove(1);
        assertEquals(3, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
    }
    
    public void testSimpleRemoveOfOriginalByIndex() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.remove(0);
        assertEquals(1, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
    }
    
    public void testRemoveOfMultipleOriginalsByIndex() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("C");
        source.add("C");
        source.add("C");
        source.add("E");
        source.add("E");
        source.add("E");
        source.remove(0);
        source.remove(3);
        source.remove(6);
        assertEquals(3, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("C", (String)(String)unique.get(1));
        assertEquals("E", (String)(String)unique.get(2));
    }
    
    public void testSimpleRemoveByObject() {
        source.add("A");
        source.remove("A");
        assertEquals(0, unique.size());
    }
    
    public void testSimpleRemoveWithMultipleUniqueValuesByObject() {
        source.add("A");
        source.add("B");
        source.add("C");
        source.remove("B");
        assertEquals(2, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("C", (String)(String)unique.get(1));
    }
    
    public void testRemoveOfASingleDuplicateByObject() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("A");
        source.remove("A");
        assertEquals(1, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
    }
    
    public void testRemoveOfAllDuplicatesByObject() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("A");
        source.remove("A");
        source.remove("A");
        source.remove("A");
        assertEquals(1, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
    }
    
    public void testRemoveOfAllValuesByObject() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("A");
        source.remove("A");
        source.remove("A");
        source.remove("A");
        source.remove("A");
        assertEquals(0, unique.size());
    }
    
    public void testRemoveOfMultipleDuplicatesByObject() {
        source.add("A");
        source.add("A");
        source.add("C");
        source.add("C");
        source.add("C");
        source.remove("A");
        source.remove("C");
        source.remove("C");
        assertEquals(2, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("C", (String)(String)unique.get(1));
    }
    
    public void testClear() {
        source.add("A");
        source.add("A");
        source.add("B");
        source.add("C");
        source.add("C");
        source.add("D");
        source.add("E");
        source.add("E");
        source.clear();
        assertEquals(0, unique.size());
    }
    
    public void testUpdateDeleteCollide() {
        source = new BasicEventList();
        source.add(new int[]{2, 0, 1});
        source.add(new int[]{2, 0, 1});
        source.add(new int[]{3, 0, 1});
        source.add(new int[]{4, 1, 0});
        IntegerArrayMatcherEditor matcherEditor = new IntegerArrayMatcherEditor(0, 0);
        FilterList filterList = new FilterList(source, matcherEditor);
        unique = new UniqueList(filterList, GlazedListsTests.intArrayComparator(0));
        matcherEditor.setFilter(2, 1);
        matcherEditor.setFilter(1, 1);
    }
    
    /**
     * Tests the change from A, B, B, D to A, C, C, D
     */
    public void testMultipleDeleteWithMultipleInsert() {
        source = new BasicEventList();
        source.add(new int[]{1, 1, 1});
        source.add(new int[]{2, 1, 0});
        source.add(new int[]{2, 1, 0});
        source.add(new int[]{3, 0, 1});
        source.add(new int[]{3, 0, 1});
        source.add(new int[]{4, 1, 1});
        IntegerArrayMatcherEditor matcherEditor = new IntegerArrayMatcherEditor(0, 0);
        FilterList filterList = new FilterList(source, matcherEditor);
        unique = new UniqueList(filterList, GlazedListsTests.intArrayComparator(0));
        matcherEditor.setFilter(1, 1);
        matcherEditor.setFilter(2, 1);
        assertEquals(3, unique.size());
        assertEquals(1, ((int[])(int[])unique.get(0))[0]);
        assertEquals(3, ((int[])(int[])unique.get(1))[0]);
        assertEquals(4, ((int[])(int[])unique.get(2))[0]);
    }
    
    /**
     * Tests the change from A, B, D to A, C, D
     */
    public void testDeleteWithInsert() {
        source = new BasicEventList();
        source.add(new int[]{1, 1, 1});
        source.add(new int[]{2, 1, 0});
        source.add(new int[]{3, 0, 1});
        source.add(new int[]{4, 1, 1});
        IntegerArrayMatcherEditor matcherEditor = new IntegerArrayMatcherEditor(0, 0);
        FilterList filterList = new FilterList(source, matcherEditor);
        unique = new UniqueList(filterList, GlazedListsTests.intArrayComparator(0));
        matcherEditor.setFilter(1, 1);
        matcherEditor.setFilter(2, 1);
        assertEquals(3, unique.size());
        assertEquals(1, ((int[])(int[])unique.get(0))[0]);
        assertEquals(3, ((int[])(int[])unique.get(1))[0]);
        assertEquals(4, ((int[])(int[])unique.get(2))[0]);
    }
    
    /**
     * Tests the change from A, B, C to C, D, E
     */
    public void testSingleValueKept() {
        source = new BasicEventList();
        source.add(new int[]{1, 1, 0});
        source.add(new int[]{2, 1, 0});
        source.add(new int[]{3, 1, 1});
        source.add(new int[]{4, 0, 1});
        source.add(new int[]{5, 0, 1});
        IntegerArrayMatcherEditor matcherEditor = new IntegerArrayMatcherEditor(0, 0);
        FilterList filterList = new FilterList(source, matcherEditor);
        unique = new UniqueList(filterList, GlazedListsTests.intArrayComparator(0));
        matcherEditor.setFilter(1, 1);
        matcherEditor.setFilter(2, 1);
        assertEquals(3, unique.size());
        assertEquals(3, ((int[])(int[])unique.get(0))[0]);
        assertEquals(4, ((int[])(int[])unique.get(1))[0]);
        assertEquals(5, ((int[])(int[])unique.get(2))[0]);
    }
    
    /**
     * Tests the change from A, A, B, B, C, C to C, C, D, D, E, E
     */
    public void testMultipleValuesKept() {
        source = new BasicEventList();
        source.add(new int[]{1, 1, 0});
        source.add(new int[]{1, 1, 0});
        source.add(new int[]{2, 1, 0});
        source.add(new int[]{2, 1, 0});
        source.add(new int[]{3, 1, 1});
        source.add(new int[]{3, 1, 1});
        source.add(new int[]{4, 0, 1});
        source.add(new int[]{4, 0, 1});
        source.add(new int[]{5, 0, 1});
        source.add(new int[]{5, 0, 1});
        IntegerArrayMatcherEditor matcherEditor = new IntegerArrayMatcherEditor(0, 0);
        FilterList filterList = new FilterList(source, matcherEditor);
        unique = new UniqueList(filterList, GlazedListsTests.intArrayComparator(0));
        matcherEditor.setFilter(1, 1);
        matcherEditor.setFilter(2, 1);
        assertEquals(3, unique.size());
        assertEquals(3, ((int[])(int[])unique.get(0))[0]);
        assertEquals(4, ((int[])(int[])unique.get(1))[0]);
        assertEquals(5, ((int[])(int[])unique.get(2))[0]);
    }
    
    /**
     * Tests the change from A, A, B, B, C, C, D, D, E, E to B, B, E, E
     */
    public void testSubset() {
        source = new BasicEventList();
        source.add(new int[]{1, 1, 0});
        source.add(new int[]{1, 1, 0});
        source.add(new int[]{2, 1, 1});
        source.add(new int[]{2, 1, 1});
        source.add(new int[]{3, 1, 0});
        source.add(new int[]{3, 1, 0});
        source.add(new int[]{4, 1, 1});
        source.add(new int[]{4, 1, 1});
        source.add(new int[]{5, 1, 0});
        source.add(new int[]{5, 1, 0});
        IntegerArrayMatcherEditor matcherEditor = new IntegerArrayMatcherEditor(0, 0);
        FilterList filterList = new FilterList(source, matcherEditor);
        unique = new UniqueList(filterList, GlazedListsTests.intArrayComparator(0));
        matcherEditor.setFilter(1, 1);
        matcherEditor.setFilter(2, 1);
        assertEquals(2, unique.size());
        assertEquals(2, ((int[])(int[])unique.get(0))[0]);
        assertEquals(4, ((int[])(int[])unique.get(1))[0]);
    }
    
    /**
     * Tests the change from A, A, B, B, C to empty to A, B, B, C, C
     */
    public void testMultipleChanges() {
        source = new BasicEventList();
        source.add(new int[]{1, 1, 0, 0});
        source.add(new int[]{1, 1, 0, 1});
        source.add(new int[]{2, 1, 0, 1});
        source.add(new int[]{2, 1, 0, 1});
        source.add(new int[]{3, 1, 0, 1});
        source.add(new int[]{3, 0, 0, 1});
        IntegerArrayMatcherEditor matcherEditor = new IntegerArrayMatcherEditor(0, 0);
        FilterList filterList = new FilterList(source, matcherEditor);
        unique = new UniqueList(filterList, GlazedListsTests.intArrayComparator(0));
        matcherEditor.setFilter(1, 1);
        matcherEditor.setFilter(2, 1);
        matcherEditor.setFilter(3, 1);
        assertEquals(3, unique.size());
        assertEquals(1, ((int[])(int[])unique.get(0))[0]);
        assertEquals(2, ((int[])(int[])unique.get(1))[0]);
        assertEquals(3, ((int[])(int[])unique.get(2))[0]);
    }
    
    /**
     * the dice for the random tests 
     */
    private Random random = new Random();
    
    /**
     * Tests a large set of random events.
     */
    public void testLargeRandomSet() {
        source = new BasicEventList();
        IntegerArrayMatcherEditor matcherEditor = new IntegerArrayMatcherEditor(0, 0);
        FilterList filterList = new FilterList(source, matcherEditor);
        unique = new UniqueList(filterList, GlazedListsTests.intArrayComparator(0));
        for (int i = 0; i < 1000; i++) {
            int value = random.nextInt(1000);
            int[] array = new int[]{value, random.nextInt(2), random.nextInt(2), random.nextInt(2)};
            source.add(array);
        }
        for (int i = 0; i < 10; i++) {
            int filterColumn = random.nextInt(3);
            matcherEditor.setFilter(filterColumn + 1, 1);
            SortedSet controlSet = new TreeSet(GlazedListsTests.intArrayComparator(0));
            controlSet.addAll(filterList);
            ArrayList controlList = new ArrayList();
            controlList.addAll(controlSet);
            Collections.sort(controlList, GlazedListsTests.intArrayComparator(0));
            assertEquals(unique.size(), controlList.size());
            for (int j = 0; j < unique.size(); j++) {
                assertEquals(((int[])(int[])unique.get(j))[0], ((int[])(int[])controlList.get(j))[0]);
            }
        }
    }
    
    /**
     * Tests a UniqueList version of a SortedList is safe when that SortedList
     * is re-sorted.
     */
    public void testReSortSource() {
        source = new BasicEventList();
        SortedList sortedList = new SortedList(source);
        unique = new UniqueList(sortedList);
        for (int i = 0; i < 1000; i++) {
            source.add(new Integer(random.nextInt(100)));
        }
        SortedSet uniqueSource = new TreeSet();
        uniqueSource.addAll(source);
        ArrayList controlList = new ArrayList();
        controlList.addAll(uniqueSource);
        assertEquals(unique, controlList);
        sortedList.setComparator(null);
        assertEquals(unique, controlList);
        sortedList.setComparator(GlazedLists.reverseComparator());
        assertEquals(unique, controlList);
    }
    
    /**
     * Test response to an UPDATE event  
     */
    public void testLeftEdgeUpdateToNewObject() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("C");
        source.add("C");
        source.add("C");
        source.add("A");
        source.set(0, "B");
        assertEquals(3, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
    }
    
    public void testLeftEdgeUpdateToLeftDuplicateObject() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("C");
        source.add("C");
        source.add("C");
        source.add("A");
        source.set(6, "A");
        assertEquals(2, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("C", (String)(String)unique.get(1));
    }
    
    public void testLeftEdgeUpdateToRightDuplicateObject() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("C");
        source.add("C");
        source.add("C");
        source.add("A");
        source.set(0, "C");
        assertEquals(2, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("C", (String)(String)unique.get(1));
    }
    
    public void testRightEdgeUpdateToNewObject() {
        ;
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("C");
        source.add("C");
        source.add("C");
        source.add("A");
        source.set(5, "B");
        assertEquals(3, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
        assertEquals("C", (String)(String)unique.get(2));
    }
    
    public void testRightEdgeUpdateToRightDuplicateObject() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("C");
        source.add("C");
        source.add("C");
        source.add("A");
        source.set(5, "C");
        assertEquals(2, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("C", (String)(String)unique.get(1));
    }
    
    public void testUniqueEndUpdateToNewObject() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("C");
        source.set(4, "B");
        assertEquals(2, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("B", (String)(String)unique.get(1));
    }
    
    public void testDuplicateEndUpdateToNewObject() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("C");
        source.add("C");
        source.set(4, "D");
        assertEquals(3, unique.size());
        assertEquals("A", (String)(String)unique.get(0));
        assertEquals("C", (String)(String)unique.get(1));
        assertEquals("D", (String)(String)unique.get(2));
    }
    
    /**
     * Verify that a unique list can be cleared.
     */
    public void testUniqueListClear() {
        unique.add("A");
        unique.add("A");
        unique.add("A");
        unique.add("B");
        unique.add("B");
        unique.add("C");
        unique.clear();
        assertEquals(0, source.size());
        assertEquals(0, unique.size());
    }
    
    /**
     * Verify that a unique list can have elements removed.
     */
    public void testUniqueListRemoveByValue() {
        unique.add("A");
        unique.add("A");
        unique.add("A");
        unique.add("B");
        unique.add("B");
        unique.add("C");
        unique.remove("B");
        assertEquals(4, source.size());
        assertEquals(2, unique.size());
    }
    
    /**
     * Verify that a unique list can have elements removed.
     */
    public void testUniqueListRemoveByIndex() {
        unique.add("A");
        unique.add("A");
        unique.add("A");
        unique.add("B");
        unique.add("B");
        unique.add("C");
        unique.remove(1);
        assertEquals(4, source.size());
        assertEquals(2, unique.size());
    }
    
    /**
     * Verify that a unique list can have elements removed.
     */
    public void testUniqueSet() {
        unique.add("A");
        unique.add("A");
        unique.add("A");
        unique.add("B");
        unique.add("B");
        unique.add("D");
        unique.set(1, "C");
        assertEquals(5, source.size());
        assertEquals(3, unique.size());
    }
    
    /**
     * Test indexOf() consistency
     */
    public void testIndexOf() {
        BasicEventList source = new BasicEventList();
        UniqueList unique = new UniqueList(source);
        Integer one = new Integer(1);
        for (int i = 0; i < 12; i++) {
            source.add(one);
        }
        Integer five = new Integer(5);
        for (int i = 0; i < 13; i++) {
            source.add(five);
        }
        Integer nine = new Integer(9);
        for (int i = 0; i < 10; i++) {
            source.add(nine);
        }
        int firstTestIndex = unique.indexOf(one);
        assertEquals(0, firstTestIndex);
        int secondTestIndex = unique.indexOf(five);
        assertEquals(1, secondTestIndex);
        int thirdTestIndex = unique.indexOf(nine);
        assertEquals(2, thirdTestIndex);
        Integer ten = new Integer(10);
        int fourthTest = unique.indexOf(ten);
        assertEquals(-1, fourthTest);
    }
    
    /**
     * Test lastIndexOf() consistency
     */
    public void testLastIndexOf() {
        BasicEventList source = new BasicEventList();
        UniqueList unique = new UniqueList(source);
        Integer one = new Integer(1);
        for (int i = 0; i < 12; i++) {
            source.add(one);
        }
        Integer five = new Integer(5);
        for (int i = 0; i < 13; i++) {
            source.add(five);
        }
        Integer nine = new Integer(9);
        for (int i = 0; i < 10; i++) {
            source.add(nine);
        }
        int firstTestIndex = unique.lastIndexOf(one);
        assertEquals(0, firstTestIndex);
        int secondTestIndex = unique.lastIndexOf(five);
        assertEquals(1, secondTestIndex);
        int thirdTestIndex = unique.lastIndexOf(nine);
        assertEquals(2, thirdTestIndex);
        Integer ten = new Integer(10);
        int fourthTest = unique.lastIndexOf(ten);
        assertEquals(-1, fourthTest);
    }
    
    /**
     * Test containment accuracy
     */
    public void testContains() {
        BasicEventList source = new BasicEventList();
        UniqueList unique = new UniqueList(source);
        Integer one = new Integer(1);
        for (int i = 0; i < 12; i++) {
            source.add(one);
        }
        Integer five = new Integer(5);
        for (int i = 0; i < 13; i++) {
            source.add(five);
        }
        Integer nine = new Integer(9);
        for (int i = 0; i < 10; i++) {
            source.add(nine);
        }
        boolean firstTest = unique.contains(one);
        assertEquals(true, firstTest);
        boolean secondTest = unique.contains(five);
        assertEquals(true, secondTest);
        boolean thirdTest = unique.contains(nine);
        assertEquals(true, thirdTest);
        Integer ten = new Integer(10);
        boolean fourthTest = unique.contains(ten);
        assertEquals(false, fourthTest);
    }
    
    /**
     * Tests that getCount() works.
     */
    public void testGetCount() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("B");
        source.add("C");
        source.add("C");
        assertEquals(3, unique.getCount(0));
        assertEquals(1, unique.getCount(1));
        assertEquals(2, unique.getCount(2));
        assertEquals(3, unique.getCount("A"));
        assertEquals(1, unique.getCount("B"));
        assertEquals(2, unique.getCount("C"));
        assertEquals(0, unique.getCount("D"));
    }
    
    /**
     * Tests that getAll() works.
     */
    public void testGetAll() {
        source.add("A");
        source.add("A");
        source.add("A");
        source.add("B");
        source.add("C");
        source.add("C");
        assertEquals(source.subList(0, 3), unique.getAll(0));
        assertEquals(source.subList(3, 4), unique.getAll(1));
        assertEquals(source.subList(4, 6), unique.getAll(2));
        assertEquals(source.subList(0, 3), unique.getAll("A"));
        assertEquals(source.subList(3, 4), unique.getAll("B"));
        assertEquals(source.subList(4, 6), unique.getAll("C"));
        assertEquals(Collections.EMPTY_LIST, unique.getAll("D"));
    }
    
    /**
     * Tests that the UniqueList can handle sets on the edge of two pairs of duplicates.
     * This was inspired by a similar test case in PopularityListTest.
     */
    public void testRightEdgeSet() {
        EventList source = new BasicEventList();
        source.add("Chevy");
        source.add("Ford");
        UniqueList uniqueList = new UniqueList(source);
        uniqueList.addListEventListener(new ListConsistencyListener(uniqueList, "unique"));
        source.set(1, "Chevy");
    }
    
    /**
     * Tests that the UniqueList can handle sets on the edge.
     */
    public void testLeftEdgeSet() {
        EventList source = new BasicEventList();
        source.add("Chevy");
        source.add("Ford");
        UniqueList uniqueList = new UniqueList(source);
        uniqueList.addListEventListener(new ListConsistencyListener(uniqueList, "unique"));
        source.set(0, "Ford");
    }
    
    /**
     * Tests that the UniqueList can handle sets on the edge.
     */
    public void testUpdateSet() {
        EventList source = new BasicEventList();
        source.add("Chevy");
        source.add("Datsun");
        source.add("Ford");
        UniqueList uniqueList = new UniqueList(source);
        uniqueList.addListEventListener(new ListConsistencyListener(uniqueList, "unique"));
        source.set(1, "Dodge");
    }
    
    /**
     * Tests that the UniqueList can handle sets on the edge.
     */
    public void testUpdateLeftSet() {
        EventList source = new BasicEventList();
        source.add("Chevy");
        source.add("Datsun");
        source.add("Ford");
        UniqueList uniqueList = new UniqueList(source);
        uniqueList.addListEventListener(new ListConsistencyListener(uniqueList, "unique"));
        source.set(1, "Chevy");
    }
    
    /**
     * Tests that the UniqueList can handle sets on the edge.
     */
    public void testUpdateRightSet() {
        EventList source = new BasicEventList();
        source.add("Chevy");
        source.add("Datsun");
        source.add("Ford");
        UniqueList uniqueList = new UniqueList(source);
        uniqueList.addListEventListener(new ListConsistencyListener(uniqueList, "unique"));
        source.set(1, "Ford");
    }
    
    /**
     * Tests that the UniqueList can handle sets on the edge.
     */
    public void testLeftUpdateSet() {
        EventList source = new BasicEventList();
        source.add("Chevy");
        source.add("Chevy");
        source.add("Ford");
        UniqueList uniqueList = new UniqueList(source);
        uniqueList.addListEventListener(new ListConsistencyListener(uniqueList, "unique"));
        source.set(1, "Chevy");
    }
    
    /**
     * Tests that the UniqueList can handle sets on the edge.
     */
    public void testLeftInsertSet() {
        EventList source = new BasicEventList();
        source.add("Chevy");
        source.add("Chevy");
        source.add("Ford");
        UniqueList uniqueList = new UniqueList(source);
        uniqueList.addListEventListener(new ListConsistencyListener(uniqueList, "unique"));
        source.set(1, "Datsun");
    }
    
    /**
     * Tests that the UniqueList can handle sets on the edge.
     */
    public void testLeftMoveSet() {
        EventList source = new BasicEventList();
        source.add("Chevy");
        source.add("Chevy");
        source.add("Ford");
        UniqueList uniqueList = new UniqueList(source);
        uniqueList.addListEventListener(new ListConsistencyListener(uniqueList, "unique"));
        source.set(1, "Ford");
    }
    
    /**
     * Tests that the UniqueList can handle sets on the edge.
     */
    public void testRightUpdateSet() {
        EventList source = new BasicEventList();
        source.add("Chevy");
        source.add("Ford");
        source.add("Ford");
        UniqueList uniqueList = new UniqueList(source);
        uniqueList.addListEventListener(new ListConsistencyListener(uniqueList, "unique"));
        source.set(1, "Ford");
    }
    
    /**
     * Tests that the UniqueList can handle sets on the edge.
     */
    public void testRightInsertSet() {
        EventList source = new BasicEventList();
        source.add("Chevy");
        source.add("Ford");
        source.add("Ford");
        UniqueList uniqueList = new UniqueList(source);
        uniqueList.addListEventListener(new ListConsistencyListener(uniqueList, "unique"));
        source.set(1, "Datsun");
    }
    
    /**
     * Tests that the UniqueList can handle sets on the edge.
     */
    public void testRightMoveSet() {
        EventList source = new BasicEventList();
        source.add("Chevy");
        source.add("Ford");
        source.add("Ford");
        UniqueList uniqueList = new UniqueList(source);
        uniqueList.addListEventListener(new ListConsistencyListener(uniqueList, "unique"));
        source.set(1, "Chevy");
    }
    
    public void testRemoveAPair() {
        EventList source = new BasicEventList();
        UniqueList uniqueList = new UniqueList(source);
        uniqueList.addListEventListener(new ListConsistencyListener(uniqueList, "UniqueList"));
        source.addAll(GlazedListsTests.stringToList("AABBBD"));
        assertEquals(GlazedListsTests.stringToList("ABD"), uniqueList);
        source.remove(0);
        source.remove(0);
        assertEquals(GlazedListsTests.stringToList("BD"), uniqueList);
    }
    
    public void testDispose() {
        assertEquals(1, source.updates.getListEventListeners().size());
        unique.dispose();
        assertEquals(0, source.updates.getListEventListeners().size());
    }
}
