package ca.odell.glazedlists.swing;

import java.awt.*;
import ca.odell.glazedlists.BasicEventList;
import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.GlazedLists;
import ca.odell.glazedlists.gui.TableFormat;

/**
 * Test EventTableModel from the Swing thread.
 *
 * @author <a href="mailto:jesse@swank.ca">Jesse Wilson</a>
 */
public class EventTableModelTest extends SwingTestCase {
    
    public EventTableModelTest() {
        super();
    }
    
    /**
     * Verifies that the new getElementAt() method of EventTableModel works.
     */
    public void guiTestGetElementAt() {
        EventList colors = new BasicEventList();
        colors.add(Color.red);
        colors.add(Color.green);
        colors.add(Color.blue);
        TableFormat colorTableFormat = GlazedLists.tableFormat(new String[]{"red", "green", "blue"}, new String[]{"Red", "Green", "Blue"});
        EventTableModel tableModel = new EventTableModel(colors, colorTableFormat);
        assertEquals(Color.red, tableModel.getElementAt(0));
        assertEquals(Color.green, tableModel.getElementAt(1));
        assertEquals(Color.blue, tableModel.getElementAt(2));
        try {
            tableModel.getElementAt(100);
            fail("failed to receive IndexOutOfBoundsException for invalid index");
        } catch (IndexOutOfBoundsException e) {
        }
        try {
            tableModel.getElementAt(-1);
            fail("failed to receive IndexOutOfBoundsException for invalid index");
        } catch (IndexOutOfBoundsException e) {
        }
    }
    
    /**
     * Verifies that the getValueAt() method of EventTableModel works.
     */
    public void guiTestGetValueAt() {
        EventList colors = new BasicEventList();
        colors.add(Color.red);
        colors.add(Color.green);
        colors.add(Color.blue);
        TableFormat colorTableFormat = GlazedLists.tableFormat(new String[]{"red", "green", "blue"}, new String[]{"Red", "Green", "Blue"});
        EventTableModel tableModel = new EventTableModel(colors, colorTableFormat);
        assertEquals(new Integer(Color.red.getRed()), tableModel.getValueAt(0, 0));
        assertEquals(new Integer(Color.green.getGreen()), tableModel.getValueAt(1, 1));
        assertEquals(new Integer(Color.blue.getBlue()), tableModel.getValueAt(2, 2));
        try {
            tableModel.getValueAt(100, 0);
            fail("failed to receive IndexOutOfBoundsException for invalid index");
        } catch (IndexOutOfBoundsException e) {
        }
        try {
            tableModel.getValueAt(-1, 0);
            fail("failed to receive IndexOutOfBoundsException for invalid index");
        } catch (IndexOutOfBoundsException e) {
        }
    }
}
